use leptos::*;
use leptos_meta::*;
use leptos_router::*;

// Modules
mod components;
mod pages;
mod util;

// Top-Level pages
use crate::pages::home::Home;
use crate::pages::not_found::NotFound;
use crate::pages::participants;
use crate::pages::times;

/// An app router which renders the homepage and handles 404's
#[component]
pub fn App() -> impl IntoView {
    // Provides context that manages stylesheets, titles, meta tags, etc.
    provide_meta_context();
    util::surrealdb::init_surrealdb();

    let websocket = expect_context::<util::surrealdb::SurrealContext>();
    let participants = use_context::<util::surrealdb::ParticipantsContext>()
        .expect("Could not find participants context");

    view! {

        // injects info into HTML tag from application code
        <Html
            lang="en"
            dir="ltr"
            attr:data-theme="light"
        />

        // sets the document title
        <Title text="WRB Timings"/>

        // injects metadata in the <head> of the page
        <Meta charset="UTF-8" />
        <Meta name="viewport" content="width=device-width, initial-scale=1.0" />


        <ErrorBoundary
            fallback=|errors| view! {
                <h1>"Uh oh! Something went wrong!"</h1>

                <p>"Errors: "</p>
                // Render a list of errors as strings - good for development purposes
                <ul>
                    {move || errors.get()
                        .into_iter()
                        .map(|(_, e)| view! { <li>{e.to_string()}</li>})
                        .collect_view()
                    }
                </ul>
            }
        >
            <Router>
                <components::navbar::Navbar />
                <p>{websocket.message}</p>
                <p>{ format!("{:?}", participants.0.get() )}</p>
                <button on:click=move |_| { websocket.send("BOE!") }>"Test Message"</button>
                <main>
                    <Routes>
                        <Route path="/" view=Home />
                        <Route path="/participants" view=participants::Participants />
                        <Route path="/participants/add" view=participants::add::Add />
                        <Route path="/times" view=times::Times />
                        <Route path="/*" view=NotFound />
                    </Routes>
                </main>
            </Router>

        </ErrorBoundary>
    }
}
