use leptos::*;
use leptos_use::{core::ConnectionReadyState, use_websocket, UseWebsocketReturn};
use std::rc::Rc;

#[derive(Clone, Debug)]
pub enum HourKind {
    A,
    B,
    C,
    D,
    Z,
}

#[derive(Clone, Debug)]
pub struct GroupKind {
    hour: HourKind,
    lane: u8,
}

#[derive(Clone, Debug)]
pub struct Participant {
    name: String,
    group: GroupKind,
}

#[derive(Clone, Debug)]
pub struct ParticipantsContext(pub ReadSignal<Vec<Participant>>);

#[derive(Clone)]
pub struct SurrealContext {
    pub message: Signal<Option<String>>,
    send: Rc<dyn Fn(&str)>,
}

impl SurrealContext {
    pub fn new(message: Signal<Option<String>>, send: Rc<dyn Fn(&str)>) -> Self {
        Self { message, send }
    }

    #[inline(always)]
    pub fn send(&self, message: &str) {
        (self.send)(message)
    }
}

pub fn init_surrealdb() {
    let UseWebsocketReturn {
        ready_state,
        message,
        send,
        ..
    } = use_websocket("wss://echo.websocket.events/");

    let (participants, _set_participants) = create_signal::<Vec<Participant>>(vec![Participant {
        name: "User1".to_string(),
        group: GroupKind {
            hour: HourKind::A,
            lane: 1,
        },
    }]);

    provide_context(SurrealContext::new(message, Rc::new(send.clone())));
    provide_context(ParticipantsContext(participants));

    let _status = move || ready_state.get().to_string();

    let _connected = move || ready_state.get() == ConnectionReadyState::Open;
}
